//  +====1====+====2====+====3====+====4====+====5====+====6====+====7====
//  NPM PCD2112A-QD_sample
//  copyright 2024 Nippon Pulse Motor Co.,Ltd.

// -+----1----+----2----+----3----+----4----+----5----+----6----+----7----
#include <Arduino.h>  // Framework: Arduino (MCU: ATmega328P)
#include <SPI.h>      // Library: SPIClass

// -+----1----+----2----+----3----+----4----+----5----+----6----+----7----
const byte mmap[] = { // memory map
    0x00, 0x00,             // r-[00h(00d)]ID code (PCD2112"A" only)
    0x00, 0x00,             // --[02h(02d)]reserved
    0x00, 0x00, 0x00, 0x00, // rw[04h(04d)]RENV2: default settings
    0x00, 0x00, 0x00, 0x00, // rw[08h(08d)]RENV1: default settings
    0x00, 0x00,             // rw[0Ch(12d)]RDS: unused
    0x00, 0x00,             // rw[0Dh(14d)]RUS: unused
    0x00, 0x00,             // rw[10h(16d)]RDR: =RUR
    0xFF, 0xFF,             // rw[12h(18d)]RUR: 1.28s (65,535d)
    0x04, 0xAF,             // rw[14h(20d)]RMG: x1 (1,199d)
    0x00, 0x00, 0x00,       // rw[16h(22d)]RDP: unused
    0x00, 0x60,             // rw[19h(25d)]RFL :  96 pps (60 rpm)
    0x00, 0xC0,             // rw[1Bh(27d)]RFH1: 192 pps (120 rpm)
    0x00, 0xC0,             // rw[1Dh(29d)]RFH2: 192 pps (unused)
    0x00, 0x00, 0x00, 0x00, // rw[1Fh(31d)]RMV: unused
    0x00, 0x07,             // rw[23h(35d)]RMD: MOD=7h (switch control)
    0x43,                   // rw[25h(37d)]RCOM: STAUD1
    0x00, 0x00, 0x00, 0x00, // r-[26h(38d)]RSTS
    0x00, 0x00,             // rw[2Ah(42d)]RIST
    0x00, 0x00,             // rw[2Ch(44d)]RIOP
    0x00, 0x00, 0x00, 0x00, // r-[2Eh(46d)]RCUN
    0x00, 0x00, 0x00, 0x00, // r-[32h(50d)]RPLS(RDWC)
    0x00, 0x00              // r-[36h(54d)]RSPD
};

// -+----1----+----2----+----3----+----4----+----5----+----6----+----7----
void setup() {
  byte comm = 0x02; // SPI write command
  byte addr = 0x12; // from RUR
  int size = 20;    // from RUR to RCOM

  SPI.begin();
  SPI.beginTransaction(SPISettings(1000000, MSBFIRST, SPI_MODE3));

  delay(400); //  Delay time for TPS3801K33DCKR (reset ic)

  digitalWrite(PIN_SPI_SS, LOW);  // SS Enable
  SPI.transfer(comm); // Command send (Write)
  SPI.transfer(addr); // Address send

  for (int i = 0; i < size; i++)
    SPI.transfer(mmap[i]);  // Data send

  digitalWrite(PIN_SPI_SS, HIGH); // SS Disable
}

// -+----1----+----2----+----3----+----4----+----5----+----6----+----7----
void loop() {
  // MCU is free!
  // Motor control is only PCD2112A!
}
